@@ -1,7 +1,7 @@
 #!/usr/bin/python -u
 
 # Console Jabber Client
-# Copyright (C) 2004-2009  Jacek Konieczny
+# Copyright (C) 2004-2010  Jacek Konieczny
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as published
@@ -33,9 +33,12 @@ import logging
 import logging.config
 import codecs
 
+import dns.exception
+
 import pyxmpp.all
 import pyxmpp.jabber.all
 import pyxmpp.exceptions
+import pyxmpp.resolver
 from pyxmpp import jabber
 
 from cjc import cjclogging
@@ -50,6 +53,8 @@ from cjc import common
 from cjc import tls
 from cjc import completions
 from cjc import cjc_globals
+from cjc.plugins import PluginContainer
+from cjc.plugin import PluginBase, Configurable
 
 class Exit(Exception):
     pass
@@ -61,6 +66,7 @@ global_settings={
     "port": ("Port number to connect to", int),
     "server": ("Server address to connect to", (str, None)),
     "tls_enable": ("Enable TLS (encrypted) connections", bool),
+    "tls_verify": ("Enable TLS certificate verification", bool),
     "tls_require": ("Require TLS (encrypted) connections", bool),
     "tls_cert_file": ("Path to user certificate file", (str, None)),
     "tls_key_file": ("Path to user private key file (default: same to tls_cert_file)", (str, None)),
@@ -78,6 +84,8 @@ global_settings={
     "editor": ("Editor for message composition. Default: $EDITOR or 'vi'", (str, None)),
     "editor_encoding": ("Character encoding for edited messages. Default: locale specific", (str, None)),
     "scrollback": ("Length of the scrollback buffers (default: 500).", int, "set_scrollback"),
+    "ipv6": ("Enable IPv6 support (auto if unset).", (bool, None), "set_ipv6"),
+    "dual_stack": ("Allow IPv4 too when IPv6 is enabled.", bool, "set_dual_stack"),
 }
 
 global_theme_attrs=(
@@ -111,21 +119,20 @@ global_theme_formats=(
     ("keybinding",u"%[info]  %(key)-10s %(function)-20s %(description)s\n"),
     ("keyfunction",u"%[info]         %(function)-20s %(description)s\n"),
     ("certificate",u"  Subject: %(subject)s\n"
-            "  Issuer: %(issuer)s\n"
-            "  Serial number: %(serial_number)s\n"
-            "  Valid not before: %(not_before)s\n"
+            "%(subject_alt_name?  Subject alt name\\: %(subject_alt_name)s\n)s"
+            "%(issuer?  Issuer\\: %(issuer)s\n)s"
+            "%(serial_number?  Serial number\\: %(serial_number)s\n)s"
+            "%(not_before?  Valid not before\\: %(not_before)s\n)s"
             "  Valid not after: %(not_after)s\n"),
     ("certificate_error",u"%[warning]Server certificate failed verification.\n\n"
-        "%[info]Server has presented following certificate chain:\n"
-        "%{chain}\n"
-        "The rejected certificate is:\n"
+        "%[info]The rejected certificate is:\n"
         "%{certificate}\n"
-        "%[error]Verification failed with error #%(errnum)i: %(errdesc)s\n"),
+        "%[error]Verification failed with error: %(errdesc)s\n"),
     ("certificate_remember",u"%[info]Server '%(who)s' has presented following certificate:\n\n"
         "%{certificate}\n"
         "Verification of that certificate failed.\n"
         "Should it be accepted in future sessions anyway?\n"),
-    ("tls_error_ignored",u"%[warning]Certificate verification error #%(errnum)i:"
+    ("tls_error_ignored",u"%[warning]Certificate verification error:"
             " '%(errdesc)s' ignored - peer certificate is known as trustworthy.\n"),
     ("tls_error_not_ignored",u"%[error]Fatal certificate verification error #%(errnum)i:"
             " '%(errdesc)s' not ignored.\n"),
@@ -133,12 +140,14 @@ global_theme_formats=(
 
 
 class Application(tls.TLSMixIn,jabber.Client):
+    instance = None
     def __init__(self, base_dir, config_file = "default", theme_file = "theme", 
             home_dir = None, profile = False):
         if cjc_globals.application is not None:
             raise "An Application instance already present"
         cjc_globals.application = self
-        self.profile=profile
+        Application.instance = self
+        self.profile = profile
         tls.TLSMixIn.__init__(self)
         jabber.Client.__init__(self, disco_name="CJC", disco_type="console")
         self.__logger=logging.getLogger("cjc.Application")
@@ -151,13 +160,17 @@ class Application(tls.TLSMixIn,jabber.Client):
             "disconnect_delay":0.25,
             "autoconnect":False,
             "tls_enable":True,
+            "tls_verify":True,
             "tls_require":False,
             "keepalive": 15*60,
             "backup_config":False,
             "case_sensitive":True,
             "status_buffer_preference":1,
             "debug":False,
-            "scrollback":500}
+            "scrollback":500,
+            "ipv6": None,
+            "dual_stack": True,
+            }
         self.set_scrollback(0, self.settings['scrollback'])
         self.aliases={}
         self.available_settings=global_settings
@@ -169,10 +182,8 @@ class Application(tls.TLSMixIn,jabber.Client):
         else:
             home=os.environ.get("HOME","")
             self.home_dir=os.path.join(home,".cjc")
-        self.plugin_dirs=[os.path.join(base_dir,"plugins"),
-                    os.path.join(self.home_dir,"plugins")]
-        self.plugins={}
-        self.plugin_modules={}
+        self.plugins = PluginContainer([os.path.join(base_dir,"plugins"),
+                    os.path.join(self.home_dir,"plugins")])
         self.event_handlers={}
         self.user_info={}
         self.info_handlers={}
@@ -194,111 +205,6 @@ class Application(tls.TLSMixIn,jabber.Client):
         completions.UserCompletion(self).register("user")
         completions.CommandCompletion(self).register("command")
 
-    def _load_plugin(self,name):
-        try:
-            mod=self.plugin_modules.get(name)
-            if mod:
-                mod=reload(mod)
-            else:
-                mod=__import__(name)
-            plugin=mod.Plugin(self,name)
-            plugin.module=mod
-            plugin.sys_path=sys.path
-            self.plugins[name]=plugin
-            self.plugin_modules[name]=mod
-        except:
-            self.__logger.exception("Exception:")
-            self.__logger.info("Plugin load failed")
-
-    def load_plugin(self,name):
-        sys_path=sys.path
-        try:
-            for path in self.plugin_dirs:
-                sys.path=[path]+sys_path
-                f=os.path.join(path,name+".py")
-                if not os.path.exists(f):
-                    continue
-                if self.plugins.has_key(name):
-                    self.__logger.error("Plugin %s already loaded!" % (name,))
-                    return
-                self.__logger.info("Loading plugin %s..." % (name,))
-                self._load_plugin(name)
-                return
-            self.__logger.error("Couldn't find plugin %s" % (name,))
-        finally:
-            sys.path=sys_path
-
-    def unload_plugin(self,name):
-        try:
-            plugin=self.plugins[name]
-        except KeyError:
-            self.__logger.error("Plugin %s is not loaded" % (name,))
-            return False
-        self.__logger.info("Unloading plugin %s..." % (name,))
-        try:
-            r=plugin.unload()
-        except:
-            self.__logger.exception("Exception:")
-            r=None
-        if not r:
-            self.__logger.error("Plugin %s cannot be unloaded" % (name,))
-            return False
-        del self.plugins[name]
-        return True
-
-    def reload_plugin(self,name):
-        try:
-            plugin=self.plugins[name]
-        except KeyError:
-            self.__logger.error("Plugin %s is not loaded" % (name,))
-            return False
-        self.__logger.info("Reloading plugin %s..." % (name,))
-        try:
-            r=plugin.unload()
-        except:
-            self.__logger.exception("Exception:")
-            r=None
-        if not r:
-            self.__logger.error("Plugin %s cannot be reloaded" % (name,))
-            return False
-        sys_path=sys.path
-        sys.path=plugin.sys_path
-        try:
-            mod=reload(plugin.module)
-            plugin=mod.Plugin(self,name)
-            plugin.module=mod
-            self.plugins[name]=plugin
-        except:
-            self.__logger.exception("Exception:")
-            self.__logger.info("Plugin load failed")
-            del self.plugins[name]
-        sys.path=sys_path
-
-    def load_plugins(self):
-        sys_path=sys.path
-        try:
-            for path in self.plugin_dirs:
-                sys.path=[path]+sys_path
-                try:
-                    d=os.listdir(path)
-                except (OSError,IOError),e:
-                    self.__logger.debug("Couldn't get plugin list: %s" % (e,))
-                    self.__logger.info("Skipping plugin directory %s" % (path,))
-                    continue
-                self.__logger.info("Loading plugins from %s:" % (path,))
-                for f in d:
-                    if f[0]=="." or not f.endswith(".py"):
-                        continue
-                    name=os.path.join(f[:-3])
-                    if not self.plugins.has_key(name):
-                        self.__logger.info("  %s" % (name,))
-                        self._load_plugin(name)
-        finally:
-            sys.path=sys_path
-
-    def get_plugin(self,name):
-        return self.plugins[name]
-
     def add_event_handler(self,event,handler):
         self.lock.acquire()
         try:
@@ -488,7 +394,7 @@ class Application(tls.TLSMixIn,jabber.Client):
         logger.addHandler(self.log_hdlr)
         libxml2.registerErrorHandler(self.xml_error_handler,None)
 
-        self.load_plugins()
+        self.plugins.load_plugins()
         self.load()
         if not self.settings["jid"]:
             self.__logger.info("")
@@ -549,7 +455,7 @@ class Application(tls.TLSMixIn,jabber.Client):
         self.send_event("stream created",stream)
 
     def session_started(self):
-        for p in self.plugins.values():
+        for p in self.plugins.get_services(PluginBase):
             try:
                 p.session_started(self.stream)
             except:
@@ -636,10 +542,13 @@ class Application(tls.TLSMixIn,jabber.Client):
         self.__logger.info(u"Connecting:")
         try:
             self.connect()
-        except pyxmpp.StreamError,e:
-            self.__logger.error("Connection failed: "+str(e))
-        except (socket.error),e:
-            self.__logger.error("Connection failed: "+e.args[1])
+        except pyxmpp.StreamError, err:
+            self.__logger.error("Connection failed: " + str(err))
+        except dns.exception.DNSException, err:
+            self.__logger.error("Connection failed, couldn't resolve host name: "
+                                                            + repr(err))
+        except (socket.error), err:
+            self.__logger.error("Connection failed: " + err.args[1])
 
     def cmd_register(self, args):
         if self.stream:
@@ -766,16 +675,16 @@ class Application(tls.TLSMixIn,jabber.Client):
         self.lock.release()
         self.disconnected()
 
-    def cmd_set(self,args):
+    def cmd_set(self, args, keep_unknown = False):
         #self.__logger.debug("args: "+`args.args`)
         fvar=args.shift()
         #self.__logger.debug("fvar: "+`fvar`+" args:"+`args.args`)
         if not fvar:
-            for plugin in [None]+self.plugins.keys():
-                if plugin is None:
-                    obj=self
+            for configurable in [None] + self.plugins.get_configurables():
+                if configurable is None:
+                    obj = self
                 else:
-                    obj=self.plugins[plugin]
+                    obj = configurable
                 for var in obj.available_settings:
                     sdef=obj.available_settings[var]
                     if len(sdef)<3:
@@ -786,8 +695,8 @@ class Application(tls.TLSMixIn,jabber.Client):
                     val=obj.settings.get(var)
                     if var=="password" and val is not None:
                         val=len(val) * '*'
-                    if plugin is not None:
-                        var="%s.%s" % (plugin,var)
+                    if configurable is not None:
+                        var="%s.%s" % (configurable.settings_namespace, var)
                     if val is None:
                         self.__logger.info("%s is not set" % (var,))
                         continue
@@ -806,12 +715,18 @@ class Application(tls.TLSMixIn,jabber.Client):
         args.finish()
 
         if "." in fvar:
-            plugin,var=fvar.split(".",1)
+            namespace, var = fvar.split(".",1)
             try:
-                obj=self.plugins[plugin]
-            except KeyError:
-                self.__logger.error("Unknown category: "+plugin)
-                return
+                obj = self.plugins.get_configurable(namespace)
+            except KeyError, err:
+                self.__logger.debug(err)
+                if keep_unknown:
+                    self.__logger.warning("Unknown category: " + namespace)
+                    obj = self
+                    var = fvar
+                else:
+                    self.__logger.error("Unknown category: " + namespace)
+                    return
         elif fvar[0]==".":
             obj=self
             var=fvar[1:]
@@ -827,7 +742,12 @@ class Application(tls.TLSMixIn,jabber.Client):
                 sdef=nsdef
             descr,typ,handler=sdef
         except KeyError:
-            self.__logger.error("Unknown setting: "+fvar)
+            if keep_unknown and val is not None:
+                self.settings[fvar] = val
+                if "." not in var:
+                    self.__logger.warning("Unknown setting: "+fvar)
+            else:
+                self.__logger.error("Unknown setting: "+fvar)
             return
 
         if val is None:
@@ -911,12 +831,16 @@ class Application(tls.TLSMixIn,jabber.Client):
             return self.cmd_set(args)
 
         if "." in fvar:
-            plugin,var=fvar.split(".",1)
+            namespace, var = fvar.split(".", 1)
             try:
-                obj=self.plugins[plugin]
+                obj = self.plugins.get_configurable(namespace)
             except KeyError:
-                self.__logger.error("Unknown category: "+plugin)
-                return
+                if fvar in self.settings:
+                    obj = self
+                    var = fvar
+                else:
+                    self.__logger.error("Unknown category: " + namespace)
+                    return
         else:
             obj=self
             var=fvar
@@ -924,7 +848,12 @@ class Application(tls.TLSMixIn,jabber.Client):
         try:
             descr,typ=obj.available_settings[var][:2]
         except KeyError:
-            self.__logger.error("Unknown setting: "+fvar)
+            if var in obj.settings:
+                del obj.settings[var]
+            elif fvar in self.settings:
+                del self.settings[fvar]
+            else:
+                self.__logger.error("Unknown setting: "+fvar)
             return
 
         if typ is None:
@@ -938,6 +867,20 @@ class Application(tls.TLSMixIn,jabber.Client):
 
     def set_scrollback(self,oldval,newval):
         ui.TextBuffer.default_length = newval
+    
+    def set_ipv6(self, oldval, newval):
+        if newval:
+            if self.settings['dual_stack']:
+                pyxmpp.resolver.set_default_address_family(socket.AF_UNSPEC)
+            else:
+                pyxmpp.resolver.set_default_address_family(socket.AF_INET6)
+        else:
+            pyxmpp.resolver.set_default_address_family(socket.AF_INET)
+
+    def set_dual_stack(self, oldval, newval):
+        if self.settings.get('ipv6') is None:
+            return
+        self.set_ipv6(None, self.settings['ipv6'])
 
     def set_debug(self,oldval,newval):
         if newval:
@@ -1013,18 +956,18 @@ class Application(tls.TLSMixIn,jabber.Client):
             return 0
         os.chmod(tmpfilename, 0600);
 
-        for plugin in [None]+self.plugins.keys():
-            if plugin is None:
-                obj=self
+        for configurable in [None] + self.plugins.get_configurables():
+            if configurable is None:
+                obj = self
             else:
-                obj=self.plugins[plugin]
+                obj = configurable
             for var in obj.available_settings:
                 descr,typ=obj.available_settings[var][:2]
                 val=obj.settings.get(var)
                 if val is None:
                     continue
-                if plugin is not None:
-                    var="%s.%s" % (plugin,var)
+                if configurable is not None:
+                    var="%s.%s" % (configurable.settings_namespace, var)
                 args=ui.CommandArgs(var)
                 if type(typ) is tuple:
                     typ=typ[0]
@@ -1106,7 +1049,7 @@ class Application(tls.TLSMixIn,jabber.Client):
                 elif cmd=="set":
                     args.shift()
                     self.__logger.debug("set %r" % (args.args,))
-                    self.cmd_set(args)
+                    self.cmd_set(args, keep_unknown = True)
                 elif cmd=="bind":
                     args.shift()
                     self.__logger.debug("bind %r" % (args.args,))
@@ -1117,11 +1060,12 @@ class Application(tls.TLSMixIn,jabber.Client):
                     self.cmd_unbind(args)
                 else:
                     self.__logger.debug("set %r" % (args.args,))
-                    self.cmd_set(args)
+                    self.cmd_set(args, keep_unknown = True)
             except (ValueError,UnicodeError):
                 self.__logger.warning(
                     "Invalid config directive %r ignored" % (l,))
         f.close()
+        self.send_event("config loaded", self.settings)
         return 1
 
     def cmd_redraw(self,args):
@@ -1244,7 +1188,7 @@ class Application(tls.TLSMixIn,jabber.Client):
         if not name:
             self.__logger.error("Plugin name missing.")
             return
-        self.load_plugin(name)
+        self.plugins.load_plugin(name)
 
     def cmd_unload_plugin(self,args):
         name=args.shift()
@@ -1252,7 +1196,7 @@ class Application(tls.TLSMixIn,jabber.Client):
         if not name:
             self.__logger.error("Plugin name missing.")
             return
-        self.unload_plugin(name)
+        self.plugins.unload_plugin(name)
 
     def cmd_reload_plugin(self,args):
         name=args.shift()
@@ -1260,7 +1204,7 @@ class Application(tls.TLSMixIn,jabber.Client):
         if not name:
             self.__logger.error("Plugin name missing.")
             return
-        self.reload_plugin(name)
+        self.plugins.reload_plugin(name)
 
     def format_keytables(self,attr,params):
         r=[]
@@ -1387,7 +1331,10 @@ class Application(tls.TLSMixIn,jabber.Client):
                 try:
                     self.__logger.error(unicode(e))
                     if isinstance(e, pyxmpp.exceptions.TLSError):
-                        self.__logger.error(u"You may try disabling encryption: /set tls_enable false")
+                        self.__logger.error(
+                                u"You may try disabling encryption" 
+                                    "(/set tls_enable false) or certificate"
+                                    " verification (/set tls_verify false) ")
                     try:
                         self.stream.close()
                     except:
@@ -1543,14 +1490,19 @@ class Application(tls.TLSMixIn,jabber.Client):
         self.send_event("roster updated",jid)
 
     def stream_state_changed(self,state,arg):
+        if state in ("connecting", "connected"):
+            if ":" in arg[0]:
+                dst = "[%s]:%i" % (arg[0], arg[1])
+            else:
+                dst = "%s:%i" % (arg[0], arg[1])
         if state=="resolving":
             self.__logger.info(u"Resolving %r..." % (arg,))
         if state=="resolving srv":
             self.__logger.info(u"Resolving SRV for %r on %r..." % (arg[1],arg[0]))
         elif state=="connecting":
-            self.__logger.info(u"Connecting to %s:%i..." % (arg[0],arg[1]))
+            self.__logger.info(u"Connecting to %s..." % (dst,))
         elif state=="connected":
-            self.__logger.info(u"Connected to %s:%i." % (arg[0],arg[1]))
+            self.__logger.info(u"Connected to %s." % (dst,))
         elif state=="authenticating":
             self.__logger.info(u"Authenticating as %s..." % (unicode(arg),))
         elif state=="binding":
@@ -1674,9 +1626,20 @@ ui.CommandTable("global",10,(
         "Reloads a plugin."),
     )).install()
 
+def cmd_hpy_dump(heapy, args):
+    logging.info("Dumping a snapshot to cjc.hpy...")
+    heapy.heap().dump("cjc.hpy")
+    logging.info(" ...done dumping the snapshot")
+ui.CommandTable("hpy",10,(
+    ui.Command("hpy_dump", cmd_hpy_dump,
+        "/hpy_dump",
+        "Dump a memory profiling snapshot to cjc.hpy",
+        ()),
+    )).install()
+
 def usage():
     print
-    print "Console Jabber Client (c) 2003-2009 Jacek Konieczny <jajcus@jajcus.net>"
+    print "Console Jabber Client (c) 2003-2010 Jacek Konieczny <jajcus@jajcus.net>"
     print
     print "Usage:"
     print "  %s [OPTIONS]" % (sys.argv[0],)
@@ -1703,10 +1666,18 @@ def usage():
     print "  -P"
     print "  --profile                Write profiling statistics"
 
-def main(base_dir,profile=False):
+def main(base_dir, profile = False, heapy = None):
+    if heapy:
+        try:
+            os.unlink("cjc.hpy")
+        except OSError:
+            pass
+        print >>sys.stderr, "Getting initial heap statistics..."
+        heapy.heap().dump("cjc.hpy")
+        libxml2.debugMemory(1)
+
     logger=logging.getLogger()
     logger.setLevel(logging.DEBUG)
-    libxml2.debugMemory(1)
     locale.setlocale(locale.LC_ALL,"")
     encoding=locale.getlocale()[1]
     if not encoding:
@@ -1755,10 +1726,47 @@ def main(base_dir,profile=False):
     app=Application(base_dir,config_file,theme_file,home_dir=config_dir,profile=profile)
     try:
         screen=ui.init()
+        if heapy:
+            ui.activate_cmdtable("hpy", heapy)
         app.run(screen)
     finally:
         #logging.debug("Cleaning up")
         ui.deinit()
         #logging.debug("Cleaned up")
+    if heapy:
+        print >>sys.stderr, "Getting pre-cleanup memory usage information..."
+        libxml2_usage = libxml2.debugMemory(1)
+        print >>sys.stderr, "  libxml2 used: {0} B".format(libxml2_usage)
+        if libxml2_usage:
+            try:
+                os.unlink(".memdump")
+            except OSError:
+                pass
+            libxml2.dumpMemory()
+            if os.path.exists(".memdump"):
+                os.rename(".memdump", "cjc-pre.memdump")
+                print >>sys.stderr, ("  libxml2 memory dump saved to"
+                                                        " 'cjc-pre.memdump'")
+        heapy.heap().dump("cjc.hpy")
+        print >>sys.stderr, "Heapy profile saved to 'cjc.hpy'"
+        print >>sys.stderr, "cleaning up..."
+        del screen
+        del app
+        libxml2.cleanupParser()
+        print >>sys.stderr, "Getting post-cleanup memory usage information..."
+        libxml2_usage = libxml2.debugMemory(1)
+        print >>sys.stderr, "  libxml2 leaked: {0} B".format(libxml2_usage)
+        if libxml2_usage:
+            try:
+                os.unlink(".memdump")
+            except OSError:
+                pass
+            libxml2.dumpMemory()
+            if os.path.exists(".memdump"):
+                os.rename(".memdump", "cjc-post.memdump")
+                print >>sys.stderr, ("  libxml2 memory dump saved to"
+                                                        " 'cjc-post.memdump'")
+        heapy.heap().dump("cjc.hpy")
+        print >>sys.stderr, "Heapy profile saved to 'cjc.hpy'"
 
 # vi: sts=4 et sw=4
