@@ -1,5 +1,5 @@
 # Console Jabber Client
-# Copyright (C) 2004-2009  Jacek Konieczny
+# Copyright (C) 2004-2010 Jacek Konieczny
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as published
@@ -445,7 +445,10 @@ class ThemeManager:
                 form=form.encode(self.encoding,"replace")
                 formatted=val.strftime(form)
             else:
-                formatted=val.strftime("%H:%M")
+                if val.date() == datetime.date.today():
+                    formatted=val.strftime("%H:%M")
+                else:
+                    formatted=val.strftime("%Y-%m-%d %H:%M")
             params[key]=unicode(formatted,self.encoding,"replace")
             return format
         elif typ==u"J":
