@@ -1,5 +1,5 @@
 # Console Jabber Client
-# Copyright (C) 2004-2009  Jacek Konieczny
+# Copyright (C) 2004-2010 Jacek Konieczny
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as published
@@ -304,6 +304,7 @@ class TextInput(InputWidget):
         p=self.content.rfind(" ",0,self.pos)
         if p<=0:
             self.content=self.content[self.pos:]
+            self.pos = 0
         else:
             self.content=self.content[:p+1]+self.content[self.pos:]
             self.pos=p+1
@@ -464,7 +465,7 @@ ktb=keytable.KeyTable("text-input",50,(
         KeyFunction("delete-char",
                 TextInput.key_del,
                 "Delete current character",
-                ("DC","^D","\x7f")),
+                ("DC","^D")),
         KeyFunction("kill-line",
                 TextInput.key_kill,
                 "Erase whole text",
