@@ -1,5 +1,5 @@
 # Console Jabber Client
-# Copyright (C) 2004-2009  Jacek Konieczny
+# Copyright (C) 2004-2010 Jacek Konieczny
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as published
@@ -20,15 +20,22 @@ import os
 import locale
 import tempfile
 import re
+import uuid
+
+from datetime import datetime
+import logging
+
+logger = logging.getLogger("cjc.plugin.message")
 
 import pyxmpp
 from cjc import ui
-from cjc.plugin import PluginBase
+from cjc.plugin import PluginBase, Archiver, Archive
 from cjc import common
 from cjc import cjc_globals
 from pyxmpp.jabber import delay
 
 theme_attrs=(
+    ("message.info", curses.COLOR_GREEN,curses.COLOR_BLACK,curses.A_NORMAL, curses.A_NORMAL),
     ("message.date", curses.COLOR_YELLOW,curses.COLOR_BLACK,curses.A_BOLD, curses.A_UNDERLINE),
     ("message.subject", curses.COLOR_YELLOW,curses.COLOR_BLACK,curses.A_BOLD, curses.A_UNDERLINE),
     ("message.sender", curses.COLOR_YELLOW,curses.COLOR_BLACK,curses.A_BOLD, curses.A_UNDERLINE),
@@ -36,6 +43,7 @@ theme_attrs=(
 )
 
 theme_formats=(
+    ("message.archive_end", "%[message.info]----- message archive end -----"),
     ("message.received",
 u"""------------------
 %[message.date]Date:    %(T:timestamp:%c)s
@@ -264,17 +272,64 @@ Subject: %(subject)s
         self.buffer.append_themed("warning",msg)
         self.buffer.update()
 
-class MessageBuffer:
+
+class MessagesBuffer(ui.TextBuffer):
+    def __init__(self, conversation):
+        self.conversation = conversation
+        try:
+            self.archive = cjc_globals.application.plugins.get_service(Archive)
+        except KeyError:
+            self.archive = None
+        if conversation.peer:
+            ui.TextBuffer.__init__(self, {"peer": conversation.peer},
+                "message.descr-per-user","message buffer", conversation)
+        else:
+            ui.TextBuffer__init__(self, {}, "message.descr",
+                                    "message buffer", conversation)
+        self.last_record = None
+
+    def fill_top_underflow(self, lines_needed):
+        if not self.archive:
+            return
+        if self.last_record:
+            older_than = self.last_record
+        else:
+            older_than = self.conversation.start_time
+        record_iter = self.archive.get_records('message', 
+                        self.conversation.peer.bare(),
+                        older_than = older_than, limit = lines_needed,
+                        order = Archive.REVERSE_CHRONOLOGICAL)
+        records = list(record_iter)
+        if not records:
+            return
+        records.reverse()
+        logger.debug("Got {0} records:".format(len(records)))
+        for record_id, record in records:
+            logger.debug("Record {0!r}: {1!r}".format(record_id, record))
+            fparams = dict()
+            if record.direction == "in":
+                fparams["from"] = record.peer
+                theme_fmt = "message.received"
+            else:
+                fparams["to"] = record.peer
+                theme_fmt = "message.sent"
+            if record.timestamp:
+                fparams["timestamp"] = record.timestamp
+            fparams["subject"] = record.subject
+            fparams["thread"] = record.thread
+            fparams["body"] = record.body
+            self.append_themed(theme_fmt, fparams)
+        self.append_themed("message.archive_end", {})
+        self.last_record = records[0][0]
+
+
+class Conversation:
     def __init__(self,plugin,peer,thread):
+        self.start_time = None
         self.plugin=plugin
         self.peer=peer
         self.thread=thread
-        if peer:
-            self.buffer=ui.TextBuffer({"peer":self.peer},
-                    "message.descr-per-user","message buffer",self)
-        else:
-            self.buffer=ui.TextBuffer({},"message.descr",
-                    "message buffer",self)
+        self.buffer = MessagesBuffer(self)
         self.buffer.preference=plugin.settings["buffer_preference"]
         self.buffer.update()
         self.last_sender=None
@@ -291,6 +346,10 @@ class MessageBuffer:
                 }
         if timestamp:
             d["timestamp"]=timestamp
+         
+        if not self.start_time:
+            self.start_time = timestamp if timestamp else datetime.now()
+
         self.buffer.append_themed("message.received",d)
         self.buffer.update()
         self.last_sender=sender
@@ -299,6 +358,8 @@ class MessageBuffer:
         self.last_thread=thread
 
     def add_sent(self,recipient,subject,body,thread):
+        if not self.start_time:
+            self.start_time = datetime.now()
         self.buffer.append_themed("message.sent",{
                 "to": recipient,
                 "subject": subject,
@@ -325,8 +386,8 @@ class MessageBuffer:
             key=self.peer.bare().as_unicode()
         else:
             key=None
-        if self.plugin.buffers.has_key(key):
-            l=self.plugin.buffers[key]
+        if self.plugin.conversations.has_key(key):
+            l=self.plugin.conversations[key]
             if self in l:
                 l.remove(self)
         self.buffer.close()
@@ -364,11 +425,12 @@ class MessageBuffer:
 
         self.plugin.send_message(self.last_sender,subject,body,self.last_thread)
 
+
 ui.CommandTable("message buffer",50,(
-    ui.Command("close", MessageBuffer.cmd_close,
+    ui.Command("close", Conversation.cmd_close,
         "/close",
         "Closes current chat buffer"),
-    ui.Command("reply", MessageBuffer.cmd_reply,
+    ui.Command("reply", Conversation.cmd_reply,
         "/reply [-subject subject] [text]",
         "Reply to the last message in window",
         ("-subject opaque","text")),
@@ -377,17 +439,13 @@ ui.CommandTable("message buffer",50,(
 class Plugin(PluginBase):
     def __init__(self,app,name):
         PluginBase.__init__(self,app,name)
-        self.buffers={}
-        self.last_thread=0
+        self.conversations={}
         cjc_globals.theme_manager.set_default_attrs(theme_attrs)
         cjc_globals.theme_manager.set_default_formats(theme_formats)
         self.available_settings={
             "buffer": ("How received messages should be put in buffers"
                     " (single|separate|per-user|per-thread)",
                     ("single","separate","per-user","per-thread")),
-            "log_filename": ("Where messages should be logged to",(str,None)),
-            "log_format_in": ("Format of incoming message log entries",(str,None)),
-            "log_format_out": ("Format of outgoing message log entries",(str,None)),
             "buffer_preference": ("Preference of message buffers when switching to the next active buffer. If 0 then the buffer is not even shown in active buffer list.",int),
             "auto_popup": ("When enabled each new message buffer is automatically made active.",bool),
             "editor": ("Editor for message composition. Default: global 'editor' option, $EDITOR or 'vi'",str),
@@ -395,13 +453,6 @@ class Plugin(PluginBase):
             }
         self.settings={
                 "buffer":"per-user",
-                "log_filename": u"%($HOME)s/.cjc/logs/messages/%(J:peer:bare)s",
-                "log_format_in": u"[%(T:now:%c)s] Incoming message\n"
-                        "From: %(sender)s\n"
-                        "Subject: %(subject)s\n%(body)s\n",
-                "log_format_out": u"[%(T:now:%c)s] Outgoing message\n"
-                        "To: %(recipient)s\n"
-                        "Subject: %(subject)s\n%(body)s\n",
                 "buffer_preference": 50,
                 "auto_popup": False,
                 }
@@ -444,81 +495,83 @@ class Plugin(PluginBase):
         composer=Composer(self)
         return composer.start(recipient,subject,body)
 
-    def send_message(self,recipient,subject,body,thread=0,buff=None):
+    def send_message(self,recipient,subject,body,thread=0,conv=None):
         if thread==0:
-            self.last_thread+=1
-            thread="message-thread-%i" % (self.last_thread,)
+            thread = unicode(uuid.uuid4())
         m=pyxmpp.Message(to_jid=recipient,stanza_type="normal",subject=subject,body=body,thread=thread)
         self.cjc.stream.send(m)
-        if buff is None:
-            buff=self.find_or_make(recipient,thread)
-        if self.settings.get("log_filename"):
-            self.log_message("out",self.cjc.jid,recipient,subject,body,thread)
-        buff.add_sent(recipient,subject,body,thread)
+        if conv is None:
+            conv=self.find_or_make(recipient,thread)
+
+        conv.add_sent(recipient,subject,body,thread)
+
+        archivers = self.cjc.plugins.get_services(Archiver)
+        for archiver in archivers:
+            archiver.log_event("message", recipient, 'out', None, subject, body, thread)
 
     def ev_presence_changed(self,event,arg):
         key=arg.bare().as_unicode()
-        if not self.buffers.has_key(key):
+        if not self.conversations.has_key(key):
             return
-        for buff in self.buffers[key]:
-            if buff.peer==arg or buff.peer==arg.bare():
-                buff.buffer.update()
+        for conv in self.conversations[key]:
+            if conv.peer==arg or conv.peer==arg.bare():
+                conv.buffer.update()
 
     def ev_day_changed(self,event,arg):
-        for buffers in self.buffers.values():
-            for buf in buffers:
-                buf.buffer.append_themed("message.day_change",{},activity_level=0)
-                buf.buffer.update()
+        for conversations in self.conversations.values():
+            for conv in conversations:
+                conv.buffer.append_themed("message.day_change",{},activity_level=0)
+                conv.buffer.update()
 
     def session_started(self,stream):
         self.cjc.stream.set_message_handler("normal",self.message_normal)
         self.cjc.stream.set_message_handler("error",self.message_error,None,90)
 
-    def find_buffer(self,user,thread):
-        buff=None
+    def find_conversation(self,user,thread):
+        conv=None
         if user:
             key=user.bare().as_unicode()
         else:
             key=user
-        if self.buffers.has_key(key):
-            buffs=self.buffers[key]
+        if self.conversations.has_key(key):
+            buffs=self.conversations[key]
             for b in buffs:
                 if thread==b.thread:
-                    buff=b
+                    conv=b
                     break
-        return buff
+        return conv
 
     def find_or_make(self,user,thread):
         bset=self.settings["buffer"]
         if bset=="separate":
             pass
         elif bset=="per-thread":
-            buff=self.find_buffer(user,thread)
-            if buff:
-                return buff
+            conv=self.find_conversation(user,thread)
+            if conv:
+                return conv
         elif bset=="per-user":
-            buff=self.find_buffer(user,None)
-            if buff:
-                return buff
+            conv=self.find_conversation(user,None)
+            if conv:
+                return conv
             thread=None
         else:
-            buff=self.find_buffer(None,None)
-            if buff:
-                return buff
+            conv=self.find_conversation(None,None)
+            if conv:
+                return conv
             thread=None
             user=None
-        buff=MessageBuffer(self,user,thread)
+        conv=Conversation(self,user,thread)
         if user:
             key=user.bare().as_unicode()
         else:
             key=user
-        if not self.buffers.has_key(key):
-            self.buffers[key]=[buff]
+        if not self.conversations.has_key(key):
+            self.conversations[key]=[conv]
         else:
-            self.buffers[key].append(buff)
+            self.conversations[key].append(conv)
         if self.settings.get("auto_popup"):
-            cjc_globals.screen.display_buffer(buff.buffer)
-        return buff
+            cjc_globals.screen.display_buffer(conv.buffer)
+        return conv
 
     def message_error(self,stanza):
         if self.settings["buffer"]=="separate":
@@ -526,17 +579,17 @@ class Plugin(PluginBase):
         fr=stanza.get_from()
         thread=stanza.get_thread()
 
-        buff=self.find_buffer(fr,thread)
+        conv=self.find_conversation(fr,thread)
         bset=self.settings["buffer"]
-        if not buff and bset in ("per-thread","per-user","single"):
-            buff=self.find_buffer(fr,thread)
-        if not buff and bset in ("per-user","single"):
-            buff=self.find_buffer(fr,None)
-        if not buff and bset=="single":
-            buff=self.find_buffer(None,None)
-        if not buff:
+        if not conv and bset in ("per-thread","per-user","single"):
+            conv=self.find_conversation(fr,thread)
+        if not conv and bset in ("per-user","single"):
+            conv=self.find_conversation(fr,None)
+        if not conv and bset=="single":
+            conv=self.find_conversation(None,None)
+        if not conv:
             return 0
-        buff.error(stanza)
+        conv.error(stanza)
         return 1
 
     def message_normal(self,stanza):
@@ -553,42 +606,16 @@ class Plugin(PluginBase):
             timestamp=d.get_datetime_local()
         else:
             timestamp=None
-        if self.settings.get("log_filename"):
-            self.log_message("in",fr,self.cjc.jid,subject,body,thread,timestamp)
-        buff=self.find_or_make(fr,thread)
-        self.cjc.send_event("message received",body)
-        buff.add_received(fr,subject,body,thread,timestamp)
-        return 1
 
-    def log_message(self,dir,sender,recipient,subject,body,thread,timestamp=None):
-        format=self.settings["log_format_"+dir]
-        filename=self.settings["log_filename"]
-        d={
-            "sender": sender,
-            "recipient": recipient,
-            "subject": subject,
-            "body": body,
-            "thread": thread
-            }
-        if dir=="in":
-            d["peer"]=sender
-        else:
-            d["peer"]=recipient
-        if timestamp:
-            d["timestamp"]=timestamp
-        filename=cjc_globals.theme_manager.substitute(filename,d)
-        s=cjc_globals.theme_manager.substitute(format,d)
-        try:
-            dirname=os.path.split(filename)[0]
-            if dirname and not os.path.exists(dirname):
-                os.makedirs(dirname)
-            f=open(filename,"a")
-            try:
-                f.write(s.encode("utf-8","replace"))
-            finally:
-                f.close()
-        except (IOError,OSError),e:
-            self.error(u"Couldn't write message log: "+unicode(e))
+        conv=self.find_or_make(fr,thread)
+        self.cjc.send_event("message received",body)
+        conv.add_received(fr,subject,body,thread,timestamp)
+
+        archivers = self.cjc.plugins.get_services(Archiver)
+        for archiver in archivers:
+            archiver.log_event("message", fr, 'in', timestamp, subject, body, thread)
+
+        return 1
 
 ui.CommandTable("message",50,(
     ui.Command("message",Plugin.cmd_message,
